
//*******************************************************************
//
//  UNIT:     G r A n i m                        02.06.99,  09.09.99
//
//  flicker free graphic animation with Delphi 4
//
//-------------------------------------------------------------------
//
//  PixelDemo1          very slow pixel drawing demo
//  LineDemo1           line drawing demo
//  AnimTilesBackground animated tile background demo
//
//*******************************************************************
unit GrAnim;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Graph, GrForm, JPEG,
  DrawRGB, Plasma, RotoZoom, Wavy, Shelly, BumpLi, Dance, Magnify;

type
  TAnimWindow = class(tMainWindow)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);

   protected
    procedure DrawBackground; override;
    procedure DrawPicture;    override;

   private
    procedure PixelDemo1;
    procedure LineDemo1 (x,y: integer);
  end;

procedure InitTileAnim;
function StartTileAnim: string;
procedure AnimTilesBackground;

var   animWindow: TAnimWindow;

//*********************************************************
implementation

type  tDemo =
      (dmTile,
       dmMagnify,
       dmDance,
       dmBumpL1,
       dmBumpL2,
       dmBumpL3,
       dmZoomRot,
       dmShelly,
       dmWavy,
       dmPlasma,
       dmRGBCol,
       dmPixel,
       dmLast);

var   tileBm: tBitmap = nil;   // background tile

//*********************************************************
procedure TAnimWindow.FormCreate(Sender: TObject);
begin
  inherited;
  SetDemos(ord(dmLast)-1);
  InitTileAnim;
end;
//*********************************************************
procedure TAnimWindow.FormDestroy(Sender: TObject);
begin
  tileBm.Free;
end;

//*********************************************************
procedure TAnimWindow.FormResize(Sender: TObject);
begin
  inherited;
  case tDemo(actDemo) of
    dmMagnify: ResizeMagnify;
    dmShelly:  ResizeShelly;
    dmBumpL1,
    dmBumpL2,
    dmBumpL3:  ResizeBumpLightning;
    dmDance:   ResizeLightningMap;
  end;
end;
//*********************************************************
procedure TAnimWindow.DrawBackground;
begin
  case tDemo(actDemo) of
    dmMagnify: caption := StartMagnify;
    dmTile:    caption := StartTileAnim;
    dmBumpL1:  caption := StartBumpLightning(1);
    dmBumpL2:  caption := StartBumpLightning(2);
    dmBumpL3:  caption := StartBumpLightning(3);
    dmDance:   caption := StartLissaDance;
    dmZoomRot: caption := StartZoomRotation;
    dmPlasma:  caption := StartPlasma;
    dmShelly:  caption := StartShelly;
    dmWavy:    caption := StartWavy;
    dmRGBCol:  caption := StartRGBColors;
    dmPixel:   PixelDemo1;
  end;
  inherited;
end;
//*********************************************************
procedure TAnimWindow.DrawPicture;
begin
  case tDemo(actDemo) of
    dmMagnify: AnimateMagnify;
    dmTile:    AnimTilesBackground;
    dmBumpL1,
    dmBumpL2,
    dmBumpL3:  BumpLightning;
    dmDance:   LissaDance;
    dmZoomRot: ZoomRotation;
    dmPlasma:  Plasma1;
    dmShelly:  Shelly1;
    dmWavy:    AnimWavy;
    else       LineDemo1(mouseX,mouseY);
  end;
  cb := #0;
end;

//*********************************************************
// example of slow pixel drawing using canvas.Pixels[..]
//---------------------------------------------------------
procedure TAnimWindow.PixelDemo1;
var   xi,yi: integer;
begin
  caption := 'slow pixel demo';
  bgBitmap.PixelFormat := pf24Bit;
  SetViewport (10,20,GetMaxX-10, GetMaxy-20, true);
  ClearViewport;
  for yi := 0 to GetMaxY
  do begin
    for xi := 0 to GetMaxX
    // draw direct to screen canvas (not buffered !!!)
    do canvas.Pixels[xi,yi] := random ($FFFFFF);
  end;
end;
//*********************************************************
// direct paint to image1 !!!
//---------------------------------------------------------
procedure TAnimWindow.LineDemo1 (x,y: integer);
var   xi: integer;
begin
  for xi := 1 to 50
  do begin
    image1.canvas.Pen.Color := random($FFFFFF);
    image1.canvas.MoveTo (x,y);
    image1.canvas.LineTo (random (pageX),random(pageY));
  end;
end;


//=========================================================
// Moving Tiles Background
//=========================================================
const tileName = 'chains1.jpg';

var   tileError: string[40] = '';

//---------------------------------------------------------
procedure InitTileAnim;
begin
  // load tiles background
  if not LoadBitmap (tileBm, tileName)
  then tileError := 'loading error picture ' + tileName
  else if tileBm = nil
  then tileError := 'creation error picture ' + tileName
  // convert to device pixel format of display device
  // to make BitBlit operations as fast as possible
  else tileBm.PixelFormat := devPixelFormat;
end;
//*********************************************************
function StartTileAnim: string;
begin
  result := 'moving tiles background';
  SetGraphMode (pfDevice);
end;
//*********************************************************
procedure AnimTilesBackground;
begin
  if tileError <> ''
  then OutTextXY (5,5, tileError)
  else DrawTilesBackground (trunc (sin (picNo / 14) * 55)
                           ,trunc (cos (picNo / 14) * 55)
                           ,tileBm, bgBitmap);
end;

end.

