//*******************************************************************
//
//  UNIT:     G r F o r m                        10.06.99,  03.09.99
//
//  main window for flicker free graphics animation with Delphi 4
//
//-------------------------------------------------------------------
//
//  tMainWindow      main window class
//
//*******************************************************************
unit GrForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Menus, JPEG,
  Graph, Palette;

type
  TMainWindow = class(TForm)
    Image1: TImage;
    PopupMenu1: TPopupMenu;
    About: TMenuItem;
    NextDemo: TMenuItem;
    Exit: TMenuItem;
    PreviousDemo: TMenuItem;
    Size2: TMenuItem;
    Size1: TMenuItem;
    Size3: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState;
                            X,Y: Integer);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure IdleHandler(Sender: TObject; var Done: Boolean);

    procedure AboutClick(Sender: TObject);
    procedure NextDemoClick(Sender: TObject);
    procedure PreviousDemoClick(Sender: TObject);
    procedure ExitClick(Sender: TObject);

    procedure SetBackgroundPicture (var bm: tBitmap);

    procedure SetDemos(no: integer);
    procedure ShowDemo(no: integer);
    procedure ShowNextDemo;
    procedure FormPaint(Sender: TObject);
    procedure Size1Click(Sender: TObject);
    procedure Size2Click(Sender: TObject);
    procedure Size3Click(Sender: TObject);
    procedure ToggleInfo;
    procedure ToggleHelp;
    procedure TogglePalette;
    procedure TogglePause;

  protected
    procedure DrawBackground; virtual;
    procedure DrawPicture;    virtual; abstract;

  private
    procedure WMEraseBkgnd(var msg: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure SetStepMode;
    procedure DrawOnePicture;
    procedure Animate;

  public
    mouseX, mouseY: integer;
    mouseButton: TMouseButton;
    maxDemo: integer;
    actDemo: integer;
  private
  end;

procedure ErrorMessage (txt: string);

//*********************************************************
implementation

uses GrAbout, GrAnim;

{$R *.DFM}

type  tInfo = (noInfo, picNoInfo, fpsInfo, maxInfo);
      tHelp = (noHelp, prgHelp, demoHelp, maxhelp);

var   showInfo:    tInfo = fpsInfo;
      showHelp:    tHelp = noHelp;
      running:     boolean = false;
      pause:       boolean = false;
      showPalette: boolean = false;
      stepMode:    boolean = false;

//*********************************************************
procedure TMainWindow.FormCreate(Sender: TObject);

begin
  // use idle handler for paint events
  //Application.OnIdle := IdleHandler;

  // set pointer to image bitmap
  imagePtr := image1.picture.bitmap;

  // init attributes
  maxDemo := 1;
  actDemo := 1;
  MarkTime;
end;
//*********************************************************
procedure TMainWindow.FormDestroy(Sender: TObject);
begin
  {needed to call TAnimWindow.FormDestroy !!!}
end;
//*********************************************************
procedure TMainWindow.FormActivate(Sender: TObject);
begin
  SetBitmapSize (clientWidth, clientHeight);
  mouseX := pageX div 2;
  mouseY := pageY div 2;
end;
//*********************************************************
// since we're going to be painting the whole form,
// handling this message will suppress the uneccessary
// repainting of the background which can result in flicker
//---------------------------------------------------------
procedure TMainWindow.WMEraseBkgnd(var msg: TWMEraseBkgnd);
begin
  msg.Result := 1;
end;
//*********************************************************
procedure TMainWindow.FormResize(Sender: TObject);
begin
  SetBitmapSize (clientWidth, clientHeight);
  image1.Width  := clientWidth;
  image1.Height := clientHeight;
  image1.picture.bitmap.width := clientWidth;
  image1.picture.bitmap.height := clientHeight;
  TextColor(clYellow);
  MarkTime;
  if stepMode then DrawOnePicture
  else if not running then Animate;
end;
//*********************************************************
procedure TMainWindow.FormMouseMove(Sender: TObject; Shift: TShiftState;
                                    X,Y: Integer);
begin
  mouseX := x;
  mouseY := y;
end;
//*********************************************************
procedure TMainWindow.FormMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mouseButton := button;
  case button of
    mbLeft: ShowNextDemo;
  end;
end;
//*********************************************************
procedure TMainWindow.TogglePause;
begin
  pause := not pause;
  stepMode := false;
  running := true;
  if pause
  then DrawOnePicture;
end;
//*********************************************************
procedure TMainWindow.ToggleInfo;
begin
  showInfo := tInfo((ord(showInfo)+1) mod ord(maxInfo));
end;
//*********************************************************
procedure TMainWindow.ToggleHelp;
begin
  showHelp := tHelp((ord(showHelp)+1) mod ord(maxHelp));
  animHelp := showHelp = demoHelp;
  if pause
  then DrawOnePicture;
end;
//*********************************************************
procedure TMainWindow.TogglePalette;
begin
  showPalette := not showPalette;
  DrawOnePicture;
end;
//*********************************************************
const vk_Ctrl_S = 83;

procedure TMainWindow.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case key of
    vk_Pause:  TogglePause;
    vk_Escape, vk_Cancel: Close;
    vk_Back: ShowDemo (actdemo - 1 + maxDemo);
    vk_F4: ;
    vk_Ctrl_S: SaveBitmap (bgBitmap, 'Grande'+intToStr(actDemo)+'.BMP');
  end;
  // show key value - only for test purpose
//  image1.Canvas.Font.Color := clBlack;
//  image1.Canvas.TextOut (46,pageY - 19,intToStr(key));
end;
//*********************************************************
procedure TMainWindow.SetStepMode;
begin
  stepMode := true;
  pause := true;
  DrawOnePicture;
end;
//*********************************************************
procedure TMainWindow.FormKeyPress(Sender: TObject; var Key: Char);
begin
  cb := key;
  case key of
    'f': ToggleInfo;
    'h': ToggleHelp;
    'c': TogglePalette;
    'p': TogglePause;
    's': SetStepMode;
    ' ': ShowNextDemo;
    else if (key >= '0') and (key <= '9') then ShowDemo (ord(key)-48);
  end;
  // show key value - only for test purpose
//  image1.Canvas.Font.Color := clBlack;
//  image1.Canvas.TextOut (46,pageY - 19,intToStr(ord(key)));
end;

//*********************************************************
procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  running := false;
end;
//*********************************************************
procedure TMainWindow.IdleHandler(Sender: TObject; var Done: Boolean);
begin
  done := false;
  DrawPicture;
end;

//*********************************************************
procedure TMainWindow.AboutClick(Sender: TObject);
begin
  AboutBox.ShowModal;
  MarkTime;
end;
//*********************************************************
procedure TMainWindow.NextDemoClick(Sender: TObject);
begin
  ShowNextDemo;
end;
//*********************************************************
procedure TMainWindow.PreviousDemoClick(Sender: TObject);
begin
  ShowDemo (actdemo - 1 + maxDemo);
end;
//*********************************************************
procedure TMainWindow.ExitClick(Sender: TObject);
begin
  Close;
end;

//*********************************************************
procedure TMainWindow.DrawBackground;
begin
  MarkTime;
end;
//*********************************************************
procedure DisplayHelp;
var   h: word;
begin
  TextColor (clYellow);
  h := TextHeight;
  ShadowTextXY (10,1*h, '  0..9   direct demo selection');
  ShadowTextXY (10,2*h, '    f       frequence / picture counter / off');
  ShadowTextXY (10,3*h, '    p      pause on/off');
  ShadowTextXY (10,4*h, '    s      picture step');
  ShadowTextXY (10,5*h, '<back>   previous demo');
  ShadowTextXY (10,6*h, '<blanc>  next demo');
  ShadowTextXY (10,7*h, ' <esc>    quit');
end;
//*********************************************************
procedure TMainWindow.DrawOnePicture;
begin
  inc (picNo);
  inc (picCnt);
  if showPalette then DrawRGBPalette (32,0,16);

  BitBlt(  image1.canvas.handle, 0,0, pageX, pageY
        ,bgBitmap.canvas.handle, 0,0, SrcCopy);

  case showHelp of
    prgHelp: DisplayHelp;
  end;

  DrawPicture;
  Invalidate;      // trigger redraw
end;
//*********************************************************
procedure TMainWindow.Animate;
var   infoText: string;
begin
  DrawBackground;
  running := true;
  repeat
    if pause
    then begin
      Sleep(100);
      MarkTime;
    end
    else begin
      DrawOnePicture;

      case showInfo of
        picNoInfo: infoText := intToStr (picCnt);
        fpsInfo:   infotext := intToStr (1000 * picCnt div RunTime);
        else infoText := '';
      end;

      // draw transparent shaded text
      TextColor (clYellow);
      ShadowTextXY (5,pageY - 20,infoText);
    end;

    application.ProcessMessages;  // !!!

  until not running;
end;

//*********************************************************
procedure TMainWindow.SetDemos(no: integer);
begin
  maxDemo := no;
end;
//*********************************************************
procedure TMainWindow.ShowDemo (no: integer);
begin
  actDemo := no mod maxDemo;
  pause := false;
  DrawBackground;
end;
//*********************************************************
procedure TMainWindow.ShowNextDemo;
begin
  ShowDemo (actDemo + 1);
end;

//*********************************************************
procedure ErrorMessage (txt: string);
begin
  Application.MessageBox (pChar(txt), 'Error', MB_ICONERROR);
  animWindow.ShowNextDemo;
end;

//*********************************************************
procedure TMainWindow.SetBackgroundPicture (var bm: tBitmap);
var rect: tRect;
    pal: tPalette;
begin
  rect.left   := 0;
  rect.top    := 0;
  rect.right  := pageX;
  rect.bottom := pageY;
  if bm.PixelFormat = pf8bit
  then begin
    GetBitmapPalette (bm, pal);
    SetAllPalette (pal);
  end;
  bgBitmap.Canvas.StretchDraw (rect, bm);
end;

//*********************************************************
procedure TMainWindow.FormPaint(Sender: TObject);
begin
  {}
end;
//*********************************************************
procedure TMainWindow.Size1Click(Sender: TObject);
begin
  clientWidth := 320;
  clientHeight := 240;
end;
//*********************************************************
procedure TMainWindow.Size2Click(Sender: TObject);
begin
  clientWidth := clientWidth * 3 div 2;
  clientHeight := clientHeight * 3 div 2;
  if top+clientHeight+54 > screenY
  then top := screenY - clientHeight-54;
end;
//*********************************************************
procedure TMainWindow.Size3Click(Sender: TObject);
begin
  clientWidth := clientWidth * 2 div 3;
  clientHeight := clientHeight * 2 div 3;
end;

end.

