//*******************************************************************
//
//  UNIT:     G r a p h                          10.06.99,  09.09.99
//
//  Good old Borland Pascal replacement unit for delphi 4.
//
//  This should only be a helpful unit for programmers who want
//  make a fast adaptation from the old Borland Pascal sources.
//  And you can look what new statements you have to use
//  instead of the old graph-unit function calls.
//
//  This unit is used only for experimental purposes.
//
//-------------------------------------------------------------------
//  SetGraphMode        select window size
//  ClearDevice         clear whole screen
//  ClearViewport       clear window
//  GetMaxX             return window width
//  GetMaxY             return window height
//  GetPixel            return pixel color
//  GetMaxY             set pixel color
//  SetAspectRatio
//  GetAspectRatio
//
//  -- additional --
//  GetStartPointer     return bitmap data starting address
//  GetLinePointer      return scanline address
//  GetPixelPointer     return pixel address
//  DisplayInfo         return display information
//  DisplayInfoString   return display information
//
//  LoadPicture         load BMP or JPG pictures
//  SavePicture         save BMP pictures
//  DrawTilesBackground fill background with tile bitmap
//
//*******************************************************************
unit Graph;

interface

uses  SysUtils, Windows, ExtCtrls, MMSystem, Graphics, JPEG,
      Dialogs, MathLib1;

{-- graph defines ---}

const { GraphResult error return codes: }
      grOk              =   0; {no error}
      grNoInitGraph     =  -1; {BGI graphics not installed}
      grNotDetected     =  -2; {graphics hardware not detected}
      grFileNotFound    =  -3; {BGI file not found}
      grInvalidDriver   =  -4; {invalide BGI file}
      grNoLoadMem       =  -5; {no mem to load BGI file}
      grNoScanMem       =  -6; {out of mem in scan fill}
      grNoFloodMem      =  -7; {out of mem in flood fill}
      grFontNotFound    =  -8; {font file not found}
      grNoFontMem       =  -9; {no mem to load font}
      grInvalidMode     = -10; {invalid graphics mode}
      grError           = -11; {graphics error}
      grIOerror         = -12; {graphics I/O error}
      grInvalidFont     = -13; {invalid font file}
      grInvalidFontNum  = -14; {invalid font number}
      grNoVesa          = -15; {no vesa bios detected}

      { colors indizes for SetPalette: }
const Black     = 0;    DarkGray     = 8;
      Blue      = 1;    LightBlue    = 9;
      Green     = 2;    LightGreen   = 10;
      Cyan      = 3;    LightCyan    = 11;
      Red       = 4;    LightRed     = 12;
      Magenta   = 5;    LightMagenta = 13;
      Brown     = 6;    Yellow       = 14;
      LightGray = 7;    White        = 15;

      {windows colors}
const clBlack        = $00000000;      cldarkGray     = $00404040;
      clBlue         = $00800000;      clLightBlue    = $00FF4040;
      clGreen        = $00008000;      clLightGreen   = $0000FF00;
      clCyan         = $00808000;      clLightCyan    = $00FFFF00;
      clRed          = $00000080;      clLightRed     = $000000FF;
      clMagenta      = $00800080;      clLightMagenta = $00FF00FF;
      clBrown        = $00008080;      clYellow       = $0000FFFF;
      clLightGray    = $00808080;      clWhite        = $00FFFFFF;

      { Fill patterns for Get/SetFillStyle: }
const EmptyFill       = 0;  { fills area in background color }
      SolidFill       = 1;  { fills area in solid fill color }
      LineFill        = 2;  { --- fill }
      LtSlashFill     = 3;  { /// fill }
      SlashFill       = 4;  { /// fill with thick lines }
      BkSlashFill     = 5;  { \\\ fill with thick lines }
      LtBkSlashFill   = 6;  { \\\ fill }
      HatchFill       = 7;  { light hatch fill }
      XHatchFill      = 8;  { heavy cross hatch fill }
      InterleaveFill  = 9;  { interleaving line fill }
      WideDotFill     = 10; { Widely spaced dot fill }
      CloseDotFill    = 11; { Closely spaced dot fill }
      UserFill        = 12; { user defined fill }

      { BitBlt operators for PutImage: }
const NormalPut     = 0;    { MOV }       { left for 1.0 compatibility }
      CopyPut       = 0;    { MOV }
      XORPut        = 1;    { XOR }
      OrPut         = 2;    { OR  }
      AndPut        = 3;    { AND }
      NotPut        = 4;    { NOT }
      XNORPut       = 5;    {additional definitions}
      NORPut        = 6;
      NANDPut       = 7;
      TransCopyPut  = 8;

const
      clipOn = true;
      clipOff = false;

type
  LineSettingsType = record
      LineStyle : word;
      Pattern   : word;
      Thickness : word;
    end;

  TextSettingsType = record
      Font      : word;
      Direction : word;
      CharSize  : word;
      Horiz     : word;
      Vert      : word;
    end;

  FillSettingsType = record               { Pre-defined fill style }
      Pattern : word;
      Color   : word;
    end;

  FillPatternType = array[1..8] of byte;  { User defined fill style }

  PointType = record
      X, Y : shortint;
    end;

  ViewPortType = record
      x1, y1, x2, y2 : integer;
      Clip           : boolean;
    end;

  ArcCoordsType = record
      X, Y           : integer;
      Xstart, Ystart : integer;
      Xend, Yend     : integer;
    end;

const GetMaxX        : integer = 639;  {max. X-Koordinate}
      GetMaxY        : integer = 479;  {max. Y-Koordinate}
      GetMaxColor    : word = 255;     {max. Farbindex}
      GetColor       : tColor = 15;    {Zeichenfarbe}
      GetBkColor     : tColor = 0;     {Hintergrundfarbe}
      GetGraphDriver : integer = 0;    {aktueller Grafikdriver}
      GetGraphMode   : integer = 0;    {aktueller Grafikmodus}
      GetTextColor     : tColor=clYellow;
      GetTextBackground: tColor=clBlack;

procedure SetGraphMode (graphMode: TPixelFormat);

{ *** Screen, viewport, page routines *** }
procedure ClearDevice;
procedure ClearViewPort;
procedure SetViewPort (x1,y1, x2,y2: integer; clip: boolean);
procedure GetViewSettings (var viewPort: viewPortType);
procedure GetAspectRatio (var Xasp, Yasp: Word);
procedure SetAspectRatio (var Xasp, Yasp: Word);

function GetX: integer;
function GetY: integer;

function  GetPixel (x,y: integer): tColor;
procedure PutPixel (x,y: integer; color: tColor);

{ *** colors *** }
procedure SetColor (color: tColor);
procedure SetBkColor (color: tColor);
procedure SetWriteMode (writeMode: integer);

{ *** polygon, fills and figures *** }
procedure Bar (x1,y1, x2,y2: integer);
procedure Rectangle (x1,y1, x2,y2: integer);

{ *** line-oriented routines *** }
procedure MoveTo (x, y: integer);
procedure MoveRel(dx,dy:integer);
procedure LineTo (x, y: integer);
procedure LineRel(dx,dy:integer);
procedure Line (x1,y1, x2,y2: integer);

{ *** text routines *** }
procedure TextColor (color: tColor);
procedure TextBackground (color: tColor);
function TextHeight: word;
procedure OutText (TextString: string);
procedure OutTextXY (x,y: integer; TextString: string);
procedure ShadowTextXY (x,y: integer; TextString: string);

{ *** additional routines *** }
procedure MarkTime;
function RunTime: longint;

procedure SetBitmapSize (xr,yr: integer);
function GetStartPointer (bm: tBitmap): pointer;
function GetLinePointer (y: integer): pointer;
function GetPixelPointer (x,y: integer): pointer;
procedure GetLine (hLine: pointer; x,y,w: integer);
procedure PutLine (hLine: pointer; x,y,w: integer);
procedure CopyLine (x1,y1,w, x2,y2: integer);
procedure CopyRect (src: tBitmap; x1,y1, x2,y2, w,h: integer);
procedure CopyBitmap (src,dest: tBitmap);

procedure DrawVLine (x,y1,y2: integer);
procedure DisplayInfo (var x,y,bpp: word);
function DisplayInfoString: string;
function LoadPicture (var pic: tPicture; filename: string): boolean;
function SavePicture (var pic: tPicture; filename: string): boolean;
function LoadBitmap (var bm: tBitmap; filename: string): boolean;
function SaveBitmap (var bm: tBitmap; filename: string): boolean;
procedure DrawTilesBackground (x,y: integer; src,dest: tBitmap);
function RGB15 (r,g,b: byte): word;
function RGB16 (r,g,b: byte): word;

var   bgBitmap: TBitmap = nil;
      imagePtr: TBitmap = nil;
      picNo:    longint = 0;           // endless counter
      picCnt:   longint = 0;           // reseted in step mode

      graphModeActive: boolean = false;
      graphError     : integer = grNoInitGraph;
      screenX        : word = 640;     // screen size
      screenY        : word = 480;
      bitsPerPixel   : word = 8;       // device data
      devPixelFormat : tPixelFormat = pf8bit;
      bytesPerPixel  : byte = 1;
      pageX          : word = 320;     // window size
      pageY          : word = 240;
      xAspect        : word = 10000;   // aspect ratio
      yAspect        : word = 10000;

      cb: char = #0;
      animHelp: boolean = false;

//*********************************************************
implementation

uses  Palette;

const activeViewPort: viewPortType   {Fensterbearbeitung}
        = (x1:   0;  y1:   0;
           x2: 640;  y2: 480;  clip: false);

var   activeRect: tRect;

      t1,t2,t3: longint;
//*********************************************************
procedure ClearDevice;

begin
  bgBitmap.Canvas.MoveTo (0,0);
  bgBitmap.Canvas.FillRect (activeRect);
end;
//*********************************************************
procedure SetViewPort (x1,y1, x2,y2: integer; clip: boolean);
begin
  activeRect.left   := x1;
  activeRect.top    := y1;
  activeRect.right  := x2+1;
  activeRect.bottom := y2+1;
  activeViewPort.clip := clip;
end;
//---------------------------------------------------------
procedure ClearViewPort;
begin
  bgBitmap.Canvas.Brush.Color := GetBkColor;
  bgBitmap.Canvas.FillRect (activeRect);
end;
//---------------------------------------------------------
procedure GetViewSettings (var viewPort: viewPortType);

begin
  viewPort := activeViewPort;
end;
//---------------------------------------------------------
procedure GetAspectRatio (var Xasp, Yasp: Word);
begin
  Xasp := xAspect;
  Yasp := yAspect;
end;
//---------------------------------------------------------
procedure SetAspectRatio (var Xasp, Yasp: Word);
begin
  xAspect := Xasp;
  yAspect := Yasp;
end;
//*********************************************************
function GetX: integer;
begin
  GetX := bgBitmap.Canvas.PenPos.x;
end;
//---------------------------------------------------------
function GetY: integer;
begin
  GetY := bgBitmap.Canvas.PenPos.y;
end;

//*********************************************************
function  GetPixel (x,y: integer): tColor;
begin
  result := bgBitmap.canvas.Pixels[x,y];
end;
//---------------------------------------------------------
procedure PutPixel (x,y: integer; color: tColor);
begin
  if bgBitmap.PixelFormat = pf8Bit
  then bgBitmap.Canvas.Pixels[x,y] := PaletteColor(byte(color))
  else bgBitmap.canvas.Pixels[x,y] := color;
end;
//*********************************************************
procedure SetColor (color: tColor);
begin
  GetColor := color;
  if bgBitmap.PixelFormat = pf8Bit
  then bgBitmap.Canvas.Pen.Color := PaletteColor(byte(color))
  else bgBitmap.Canvas.Pen.Color := color;
end;
//---------------------------------------------------------
procedure SetBkColor (color: tColor);

begin
  GetBkColor := color;
  bgBitmap.Canvas.Brush.Color := color;
end;
//---------------------------------------------------------
procedure SetWriteMode (writeMode: integer);
var   penMode: tPenMode;
begin
  case writeMode of
    CopyPut:    penMode := pmCopy;
    XORPut:     penMode := pmXor;
    else        penMode := pmCopy;
  end;
  bgBitmap.Canvas.Pen.Mode := penMode;
end;

//*********************************************************
procedure Bar (x1,y1, x2,y2: integer);
var   rect: tRect;
begin
  rect.left   := x1;
  rect.top    := y1;
  rect.right  := x2+1;
  rect.bottom := y2+1;
  bgBitmap.Canvas.FillRect(rect);
end;
//*********************************************************
procedure Rectangle (x1,y1, x2,y2: integer);
begin
  bgBitmap.Canvas.Rectangle (x1,y1, x2,y2);
end;

//*********************************************************
procedure MoveTo (x, y: integer);
begin
  bgBitmap.Canvas.MoveTo (x,y);
end;
//---------------------------------------------------------
procedure MoveRel (dx, dy: integer);
var  p: tPoint;
begin
  p := bgBitmap.Canvas.PenPos;
  inc (p.x, dx);
  inc (p.y, dy);
  bgBitmap.Canvas.MoveTo (p.x, p.y);
end;
//*********************************************************
procedure LineTo (x, y: integer);
begin
  bgBitmap.Canvas.LineTo (x, y);
end;
//---------------------------------------------------------
procedure LineRel (dx, dy: integer);
var  p: tPoint;
begin
  p := bgBitmap.Canvas.PenPos;
  inc (p.x, dx);
  inc (p.y, dy);
  bgBitmap.Canvas.LineTo (p.x, p.y);
end;
//*********************************************************
procedure Line (x1,y1, x2,y2: integer);
begin
  bgBitmap.Canvas.MoveTo (x1,y1);
  bgBitmap.Canvas.LineTo (x2,y2);
end;
//*********************************************************
procedure TextColor (color: tColor);
begin
  GetTextColor := color;
  bgBitmap.Canvas.Font.Color := color;
end;
//---------------------------------------------------------
procedure TextBackground (color: tColor);
begin
  GetTextBackground := color;
end;
//---------------------------------------------------------
function TextHeight: word;
begin
  result := 16;
end;
//---------------------------------------------------------
procedure OutText (TextString: string);
begin
  OutTextXY (GetX,GetY, TextString);
end;
//---------------------------------------------------------
procedure OutTextXY (x,y: integer; TextString: string);
begin
  SetBkMode(bgBitmap.Canvas.Handle, Transparent);
  bgBitmap.Canvas.Font.Style := [fsBold];
  bgBitmap.Canvas.TextOut (x,y, textstring);
end;
//---------------------------------------------------------
procedure ShadowTextXY (x,y: integer; TextString: string);
begin
  with imagePtr.canvas
  do begin
    SetBkMode(Handle, Transparent);
    Font.Style := [fsBold];
    Font.Color := GetTextBackground;
    TextOut (x+1,y+1, textstring);
    Font.Color := GetTextColor;
    TextOut (x,y, textstring);
  end;
end;

//=========================================================
// additional routines
//=========================================================
procedure MarkTime;
begin
  picCnt := 0;
  t1 := timeGetTime;
end;
//*********************************************************
function RunTime: longint;
begin
  t2 := timeGetTime;
  t3 := t2 - t1;
  if t3 <= 0 then t3 := 1;
  result := t3;
end;
//*********************************************************
procedure SetBitmapSize (xr,yr: integer);
begin
  bgBitmap.Width  := xr;
  bgBitmap.Height := yr;
  pageX := xr;
  pageY := yr;
  GetMaxX := pageX - 1;
  GetMaxY := pageY - 1;
  SetViewPort (0,0,GetMaxX, GetMaxY, true);
end;
//*********************************************************
function GetStartPointer (bm: tBitmap): pointer;
begin
  GetStartPointer := bm.ScanLine[bm.Height-1];
end;
//*********************************************************
function GetLinePointer (y: integer): pointer;
begin
{$ifopt D+}
  if (y < 0) or (y > GetMaxY) then RunError (231);
{$endif}
  GetLinePointer := bgBitmap.ScanLine[y];
end;
//*********************************************************
function GetPixelPointer (x,y: integer): pointer;
var      linePtr: pByteArray;
begin
{$ifopt D+}
  if (x < 0) or (x > GetMaxX) then RunError (231);
  if (y < 0) or (y > GetMaxY) then RunError (231);
{$endif}
  linePtr := bgBitmap.ScanLine[y];
  result := pointer(longint(linePtr) + x*bytesPerPixel);
end;
//*********************************************************
// copy w pixels from position x,y to horizontal line buffer
//---------------------------------------------------------
// nicht getestet ???
procedure GetLine (hLine: pointer; x,y,w: integer);
begin
{$ifopt D+}
  if (longint(x)+w) > pageX   then RunError (231);
  if (y < 0) or (y > GetMaxY) then RunError (231);
{$endif}
  Move (GetPixelPointer(x,y)^, hline^, w*bytesPerPixel);
end;
//*********************************************************
// copy w pixels of given horizontal line to position x,y
//---------------------------------------------------------
procedure PutLine (hLine: pointer; x,y,w: integer);
begin
{$ifopt D+}
  if (longint(x)+w) > pageX   then RunError (231);
  if (y < 0) or (y > GetMaxY) then RunError (231);
{$endif}
  Move (hline^, GetPixelPointer(x,y)^, w*bytesPerPixel);
end;
//*********************************************************
// copy w pixels of horizontal line from position x1,y1
// to position x2,y2
//---------------------------------------------------------
procedure CopyLine (x1,y1,w, x2,y2: integer);
begin
{$ifopt D+}
  if (longint(x2)+w) > pageX    then RunError (231);
  if (y1 < 0) or (y1 > GetMaxY) then RunError (231);
  if (y2 < 0) or (y2 > GetMaxY) then RunError (231);
{$endif}
  Move (GetPixelPointer (x1,y1)^
       ,GetPixelPointer (x2,y2)^, w*bytesPerPixel);
end;
//*********************************************************
// copy rectangle with size w*h from src at position y1
// to background bitmap at position y2
//---------------------------------------------------------
procedure CopyRect (src: tBitmap; x1,y1, x2,y2, w,h: integer);
begin
  if y1 > GetMaxY then exit;
  if y2 > GetMaxY then exit;
  if y1 < 0
  then begin
    h := h - y1;
    y2 := y2 - y1;
    y1 := 0;
  end;
  if y2 < 0
  then begin
    h := h - y2;
    y1 := y1 - y2;
    y2 := 0;
  end;
  BitBlt(bgBitmap.canvas.handle, x2,y2, w, h
             ,src.canvas.handle, x1,y1, SrcCopy);
end;
//*********************************************************
// direct bitmap copy.   only for same pixel format !!!
//---------------------------------------------------------
procedure CopyBitmap (src, dest: tBitmap);
var   yi, bpp: word;
begin
{
  // color palette error ???
  BitBlt(dest.canvas.handle, 0,0, src.Width, src.Height
        ,src.canvas.handle, 0,0, SrcCopy);
}
  case src.pixelFormat of
    pf15bit, pf16bit: bpp := 2;
    pf24bit, pf32bit: bpp := 3;
    else              bpp := 1;
  end;

  bpp :=  bpp * src.width;
  for yi := 0 to src.height-1
  do Move ( src.Scanline[yi]^
          ,dest.Scanline[yi]^, bpp);
end;
//*********************************************************
procedure DisplayInfo (var x,y,bpp: word);
var   dc: HDC;
begin
  dc  := GetDc(GetActiveWindow);
  // screen resolution
  x   := GetDeviceCaps (dc, HORZRES);
  y   := GetDeviceCaps (dc, VERTRES);
  // bits per pixel
  bpp := GetDeviceCaps (dc, BITSPIXEL)
        *GetDeviceCaps (dc, PLANES);
end;
//*********************************************************
function DisplayInfoString: string;
var   x,y,bpp: word;
begin
  DisplayInfo (x,y,bpp);
  result := Format('%dx%dx%d', [x,y,bpp]);
end;
//*********************************************************
procedure SetGraphMode (graphMode: TPixelFormat);
begin
  if graphmode = pfDevice
  then graphMode := devPixelFormat;
  case graphMode of
    pf8Bit:           bytesPerPixel := 1;
    pf15bit, pf16bit: bytesPerPixel := 2;
    pf24bit, pf32bit: bytesPerPixel := 3;
    else begin
      graphError := grInvalidMode;
      exit;
    end;
  end;

  if bgBitmap.PixelFormat <> graphMode
  then begin
    ClearViewPort;
    bgBitmap.PixelFormat := graphMode;
  end;
  xAspect := round (10000.0 * screenY * 4 / (screenX * 3));
end;

//*********************************************************
//  load BMP or JPG pictures
//---------------------------------------------------------
function LoadPicture (var pic: tPicture; filename: string): boolean;
var jp: tJPEGImage;
begin
  result := true;
  if pic = nil then pic := tPicture.Create;
  if pos('.JPG', upperCase(filename)) <> 0
  // load JPEG picture
  then begin
    jp := TJpegImage.Create;
    try
      try
        jp.LoadFromFile (filename);
        pic.bitmap.Width := jp.Width;
        pic.bitmap.Height := jp.Height;
        pic.bitmap.Canvas.Draw (0,0, jp);
      except
        result := false;
      end;
    finally
      jp.Free;
    end;
  end
  // load BMP picture
  else begin
    try
      pic.LoadFromFile (filename);
    except
      result := false;
    end;
  end;
end;
//*********************************************************
//  save BMP pictures
//---------------------------------------------------------
function SavePicture (var pic: tPicture; filename: string): boolean;
begin
  result := false;
  if pic = nil then exit;
  result := true;
  try
    pic.SaveToFile (filename);
  except
    result := false;
  end;
end;
//*********************************************************
//  load BMP or JPG pictures
//---------------------------------------------------------
function LoadBitmap (var bm: tBitmap; filename: string): boolean;
var jp: tJPEGImage;
begin
  result := true;
  if bm = nil then bm := tBitmap.Create;
  if pos('.JPG', upperCase(filename)) <> 0
  // load JPEG picture
  then begin
    jp := TJpegImage.Create;
    try
      try
        jp.LoadFromFile (filename);
        bm.Width := jp.Width;
        bm.Height := jp.Height;
        bm.Canvas.Draw (0,0, jp);
      except
        result := false;
      end;
    finally
      jp.Free;
    end;
  end
  // load BMP picture
  else begin
    try
      bm.LoadFromFile (filename);
    except
      result := false;
    end;
  end;
end;
//*********************************************************
//  save BMP pictures
//---------------------------------------------------------
function SaveBitmap (var bm: tBitmap; filename: string): boolean;
begin
  result := false;
  if bm = nil then exit;
  result := true;
  try
    bm.SaveToFile (filename);
  except
    result := false;
  end;
end;
//*********************************************************
// fill background with given tile bitmap with offset x,y
//---------------------------------------------------------
procedure DrawTilesBackground (x,y: integer;
                               src,dest: tBitmap);
var  xi,yi: integer;
begin
  while x > 0 do x := x - src.width;
  while y > 0 do y := y - src.height;
  yi := y;
  while yi < dest.height
  do begin
    xi := x;
    while xi < dest.width
    do begin
      dest.Canvas.Draw (xi,yi, src);
      xi := xi + src.width;
    end;
    yi := yi + src.height;
  end;
end;
{---------------------------------------------------------}
procedure DrawVLine (x,y1,y2: integer);
begin
  if x < 0 then exit;
  if x > GetMaxX then exit;
  with bgBitmap.Canvas
  do begin
    MoveTo (x,y1);
    LineTo (x,y2);
  end;
end;
{*********************************************************}
{ convert r g b values to word color
{ input:     r,g,b:    rgb values of pixel color (0..255)
{ return:    word:     15 bit color
{ format:    15bit (555) 0rrrrrgg gggbbbbb
{---------------------------------------------------------}
function RGB15 (r,g,b: byte): word;
begin
  Result := b shr 3 or (g shl 2) and $03E0
                    or (r shl 7) and $FC00;
end;
{*********************************************************}
{ convert r g b values to word color
{ input:     r,g,b:    rgb values of pixel color (0..255)
{ return:    word:     16 bit color
{ format:    16bit (565) rrrrrggg gggbbbbb
{---------------------------------------------------------}
function RGB16 (r,g,b: byte): word;
begin
  Result := b shr 3 or (g shl 3) and $07E0
                    or (r shl 8) and $F800;
end;
//*********************************************************
initialization
  DisplayInfo (screenX, screenY, bitsPerPixel);
  case bitsPerPixel of
    1:  devPixelFormat := pf1bit;
    4:  devPixelFormat := pf4bit;
    8:  devPixelFormat := pf8bit;
    15: devPixelFormat := pf15bit;
    16: devPixelFormat := pf16bit;
    24: devPixelFormat := pf24bit;
    32: devPixelFormat := pf32bit;
  end;
  // create background image and set parameter
  bgBitmap := TBitmap.Create;
  if bgBitmap = nil
  then raise EOutOfMemory.CreateFmt('error at creating background picture', []);
  bgBitmap.HandleType  := bmDIB;
  bgBitmap.PixelFormat := pf24Bit;
  SetBitmapSize (320,240);

//*********************************************************
finalization
  if bgBitmap <> nil then bgBitmap.Free;

end.

