{*******************************************************************}
{*                                                                 *}
{*  UNIT:          M a t h L i b 1             26.05.91, 04.11.99  *}
{*                                                                 *}
{*-----------------------------------------------------------------*}
{*  Funktion:      Diese Unit enthlt zustzliche mathematische    *}
{*                 Routinen und einige mathematische Konstanten.   *}
{*                                                                 *}
{*  -------------------------------------------------------------- *}
{*                                                                 *}
{*  MathError     Abfrage, ob mathematischer Fehler aufgetreten    *}
{*  SetMathError  Setzen eines mathematischen Fehlers              *}
{*  GetMathError  Abfragen des letzten mathematischen Fehlers      *}
{*                                                                 *}
{*  safeDiv       Division mit Prfung                             *}
{*  safeSqrt      Wurzelziehen mit Prfung                         *}
{*  safeRound     Runden auf Ganzzahl mit Prfung                  *}
{*  safeExp       Exponentialfunktion mit Prfung                  *}
{*  safeLn        natrlicher Logarithmus mit Prfung              *}
{*                                                                 *}
{*  min           Minimum 2er Zahlen                               *}
{*  max           Maximum 2er Zahlen                               *}
{*                                                                 *}
{*  XPowerY       berechnet x hoch y                               *}
{*  XPowerN       berechnet x hoch n                               *}
{*  XRootY        berechnet y-ten Wurzel aus x                     *}
{*                                                                 *}
{*  sinB, sinS    Sinustabellen                                    *}
{*  cosB, cosS    Cosinustabellen                                  *}
{*                                                                 *}
{*  cot           berechnet Cotangens eines Winkels                *}
{*  tan           berechnet Tangens eines Winkels                  *}
{*  cot           berechnet Cotangens eines Winkels                *}
{*  arcSin        berechnet Umkehrfunktion zu Sinus                *}
{*  arcCos        berechnet ArcusCosinus                           *}
{*  arcTan2       berechnet ArcusTangens mit 2 Argumenten          *}
{*  arcCot        berechnet Umkehrfunktion zu Cotangens            *}
{*                                                                 *}
{*  degree        Umrechnung von Radiant in Altgrad                *}
{*  radiant       Umrechnung von Altgrad in Radiant                *}
{*                                                                 *}
{*  exp10         berechnet von 10 hoch x                          *}
{*  log           berechnet Logarithmus von x zur Basis b          *}
{*  log10         berechnet Logarithmus von x zur Basis 10         *}
{*                                                                 *}
{*  signum        Ermittelt Vorzeichen einer reellen Zahl          *}
{*  frac          Ermittelt Nachkommateil einer reellen Zahl       *}
{*  fix           Beschneidet reelle Zahl auf n Nachkommastellen   *}
{*                                                                 *}
{*  fak           Berechnet Fakultt                               *}
{*  variations    Berechnet Variation von k aus n Elementen        *}
{*  combinations  Berechnet Kombinationen von k aus n Elementen    *}
{*                                                                 *}
{*  ggt           Berechnet grssten gemeinsamen Teiler            *}
{*  kgv           Berechnet kleinstes gemeinsames Vielfaches       *}
{*                                                                 *}
{*  fib           Berechnet Fibonacci-Zahl                         *}
{*                                                                 *}
{*******************************************************************}
{I comp_opt.inc}
{$X+}

UNIT MathLib1;

INTERFACE

{$I float.inc}

const {runtime errors}
      errorDivisionByZero         = 200;
      errorFloatingPointOverflow  = 205;
      errorFloatingPointUnderflow = 206;
      errorInvalidFloatOperation  = 207;

const {ranges of the IEEE floating point types, including denormals}
      minSingle        =     1.5e-45;
      maxSingle        =     3.4e+38;
      minDouble        =     5.0e-324;
      maxDouble        =     1.7e+308;
      minExtended      =     3.4e-4932;
      maxExtended      =     1.1e+4932;
      minComp          =     -9.223372036854775807e+18;
      maxComp          =     9.223372036854775807e+18;

const Pi14  = Pi * 0.25;          { 45 = Pi / 4}
      Pi24  = Pi * 0.50;          { 90 = Pi / 2 = 1.5707963268...}
      Pi34  = Pi * 0.75;          {135 = Pi * 3 / 4}
      {Pi = 3.1415926535897932385;{180 = Kreiszahl}
      Pi54  = Pi * 1.25;          {225 = Pi * 5 / 4}
      Pi64  = Pi * 1.50;          {270 = Pi * 3 / 2}
      Pi74  = Pi * 1.75;          {315 = Pi * 7 / 4}
      Pi84  = Pi * 2.00;          {360 = 2 * Pi = 6.2831853...}

const rad45 = Pi * 0.25;          {Radiantwerte mit Grad-Bezeichnung}
      rad90 = Pi * 0.50;
      rad135= Pi * 0.75;
      rad180= Pi;
      rad225= Pi * 1.25;
      rad270= Pi * 1.50;
      rad315= Pi * 1.75;
      rad360= Pi * 2.00;

const rad   = 180.0 / Pi;         {1 Radiant = 57.295779513...}
      deg   = Pi / 180.0;         {1 / Radiant = 0.017453292...}

      ln10  = 2.30258509299405;   {Logarithmus von 10}
      rln10 = 0.434294481903252;  {1 / ln (10)}

      sqrt2 = 1.41421356237310;   {Wurzel aus 2}
      sqrt3 = 1.73205080756888;   {Wurzel aus 3}
      sqrt5 = 2.23606797749979;   {Wurzel aus 5}

      eps = 1E-6;                 {sehr kleiner Wert}
      golden = 1.6180340052;      {Verhltniszahl goldener Schnitt}

type  tMathError        {-- mathematical errors --}
      = (mOK            {no error}
        ,mDivZero       {200: division by zero}
        ,mOverflow      {205: floating point overflow}
        ,mUnderflow     {206: floating point underflow}
        ,mInvalid       {207: invalid floating point operation}
        ,mMinFloat);    {-unendlich: grte neg. Zahl}

function MathError: boolean;
function SetMathError (mErr: tMathError; text: string): float;
function GetMathError (var mErr: tMathError; var text: string): boolean;
procedure ClearMathError;

function safeDiv (x, y: float): float;  {x/y}
function safeSqrt (x: float): float;
function safeRound (x: float): longint;
function safeExp (x: float): float;
function safeLn (x: float): float;

function XPowerY (x,y: float): float;
function XPowerN (x: float; n: integer): float;
function XRootY (x,y: float): float;

function tan (alpha: float): float;
function arcTan2 (x, y: float): float;
function arcSin (y: float): float;
function arcCos (y: float): float;
function arcCot (x: float): float;

function degree (angle: float): float;
function radiant (angle: float): float;

function exp10 (y: float): float;
function log (x,b: float): float;
function log10 (x: float): float;

function min (a, b: float): float;
function max (a, b: float): float;

function signum (x: float): float;
function frac (x: float): float;
function fix (x: float; n: word): float;

function fak (n: word): float;
function variations (k,n: word): float;
function combinations (k,n: word): float;

function ggt (n1, n2: longint): longint;
function kgv (n1, n2: longint): longint;

function Fib (n: integer): float;

{-- Sinus- Cosinus-Tabelle mit Bytewerten --}
const offSCS = 127;      {Index Nullwert}
      divSCS = 127;      {Divisionswerte -> -1.0..1.0}

var   sinB, cosB: array[0..255] of byte;     {   0..254}
      sinS, cosS: array[0..255] of shortint; {-127..127}

{*********************************************************}
IMPLEMENTATION  {uses crt2;}

const lastMathError: tMathError = mOK;
      mathErrorText: string[40] = '';

{*********************************************************}
{
{ Aufruf:      MathError (mErr, text);
{
{ Funktion:    Ermittelt, ob math. Fehler aufgetreten ist
{              und lscht letzten Fehler
{
{ Rckgabe:    MathError: boolean;   Rckgabewert
{                 =true: math. Fehler aufgetreten
{
{*********************************************************}
function MathError: boolean;

begin
  MathError := lastMathError <> mOK;
  lastMathError := mOK;
end;
{*********************************************************}
{
{ Aufruf:      SetMathError (mErr, text);
{
{ Funktion:    Setzt Mathematikfehler mErr, merkt sich text
{              und gibt Fehlerwerte je nach Versorgung
{              zurck
{
{ Versorgung:  mErr: tMathError;  Fehlerart
{              text: string;      erklrender Text
{
{ Rckgabe:    MathError: float;   Rckgabefehlwert
{                 mErr=mOverflow:  maxFloat
{                 mErr=mMinFloat:  minFloat
{                 mErr=mUnderflow: 0.0
{                 else             0.0
{
{*********************************************************}
function SetMathError (mErr: tMathError; text: string): float;

begin
  mathErrorText := text;
  lastMathError := mErr;
  if      mErr = mOverflow  then SetMathError := overflow
  else if mErr = mMinFloat  then SetMathError := minFloat
  else SetMathError := 0.0;
end;
{*********************************************************}
{
{ Aufruf:      GetMathError (mErr, text);
{
{ Funktion:    Ermittelt, ob math. Fehler aufgetreten, und
{              gibt Fehlerart und Fehlertext zurck
{
{ Ergebnis:    mErr: tMathError;  Fehlerart
{              text: string;      erklrender Text
{
{ Rckgabe:    GetMathError: boolean;   Rckgabewert
{                 =true: math. Fehler aufgetreten
{
{*********************************************************}
function GetMathError (var mErr: tMathError; var text: string): boolean;

begin
  GetMathError := lastMathError <> mOK;
  mErr := lastMathError;
  text := mathErrorText;
  lastMathError := mOK;
end;
{*********************************************************}
{ Aufruf:      ClearMathError;
{ Funktion:    letzten Mathematikfehler lschen
{*********************************************************}
procedure ClearMathError;

begin
  lastMathError := mOK;
  mathErrorText := 'OK';
end;
{*********************************************************}
{ Sicheres Dividieren zweier Gleitpunktzahlen: x / y
{---------------------------------------------------------}
function safeDiv (x, y: float): float;

begin
  if y = 0
  then safeDiv := SetMathError (mOverflow, 'DIV: y=0')
  else safeDiv := x / y;
end;
{*********************************************************}
{ Sicheres Wurzelziehen einer Gleitpunktzahl
{---------------------------------------------------------}
function safeSqrt (x: float): float;

begin
  if x > 0
  then safeSqrt := Sqrt(x)
  else safeSqrt := SetMathError (mInvalid, 'SQRT: x <= 0')
end;
{*********************************************************}
{ Sicheres Runden auf ganzzahligen Wert
{---------------------------------------------------------}
function safeRound (x: float): longint;

begin
  if x > maxLongInt
  then begin
    SetMathError (mInvalid, 'Round: x > maxLongInt');
    safeRound := maxLongInt;
  end
  else if x < -maxLongInt
  then begin
    SetMathError (mInvalid, 'Round: x < -maxLongInt');
    safeRound := -maxLongInt;
  end
  else safeRound := round(x);
end;
{*********************************************************}
{ Sicheres Berechnen der Exponentialfunktion
{---------------------------------------------------------}
function safeExp (x: float): float;

begin
  if x > maxExp
  then safeExp := SetMathError (mOverflow, 'Exp: x > maxExp')
  else if x < -maxExp
  then safeExp := 0.0
  else safeExp := exp(x);
end;
{*********************************************************}
{ Sicheres Berechnen des natrlichen Logarithmus [x <> 0]
{---------------------------------------------------------}
function safeLn (x: float): float;

begin
  if x = 0
  then safeLn := SetMathError (mMinFloat, 'LN: x=0')
  else safeLn := Ln (Abs (x));
end;
{*********************************************************}
function min (a, b: float): float;

begin
  if a < b then min := a else min := b;
end;
{*********************************************************}
function max (a, b: float): float;

begin
  if a > b then max := a else max := b;
end;
{*********************************************************}
{
{ Aufruf:      XPowerY (x,y)
{
{ Funktion:    Berechnet y^y (x hoch y)
{              (frs Potenzieren und Wurzelziehen)
{              Fr negative x muss y ganzzahlig sein.
{              Ist x = 0.0 muss y >= 0.0 sein.
{
{ Formel:      x^y = exp (y*ln(x))  [x > 0.0]
{              0^y = 0.0  [y # 0]
{              x^0 = 1.0  [x # 0]
{              0^0 = 1.0
{
{ Literatur:   Bartsch-84, Seite 53
{
{ Versorgung:  x: float;   Basiszahl
{              y: float;   Exponent
{
{ Rckgabe:    XPowerY: float;   x hoch y
{
{---------------------------------------------------------}
function XPowerY (x,y: float): float;

begin
  if      x > 0.00001 then XPowerY := +exp (y * ln(+x))
  else if x <-0.00001 then XPowerY := -exp (y * ln(-x))   {???}
  else
    if x > -0.00001
    then
      if y = 0.0
      then XPowerY := 1.0              {0 hoch 0 = 1}
      else
        if y > 0.0
        then XPowerY := 0.0            {0 hoch +y = 0}
        else XPowerY := SetMathError (mInvalid, 'XPowerY: -y by x = 0.0')
    else
      if y <> trunc (y)                {y nicht ganzzahlig ?}
      then XPowerY := SetMathError (mInvalid, 'XPowerY: for -x use only integer y')
      else begin
{
        write (x:12:4,y:16:4);
        writeln (exp (y * ln (ABS (x))):16:4);
}
        if Odd (Round (y))             {y ungerade ?}
        then XPowerY := -exp (y * ln (ABS (x)))
        else XPowerY := +exp (y * ln (ABS (x)));
      end;
end;
{*********************************************************}
function xXPowerY (x,y: float): float;

begin
  xXPowerY := 0.0;
  if      y = 0.0   then xXPowerY := 1.0
  else if x = 0.0   then xXPowerY := 0.0
    else if Abs (y * Ln (Abs (x))) > maxExp
      then RunError (errorFloatingPointOverflow)
      else if x > 0.0
        then xXPowerY := Exp (y * Ln(x))
        else if (x < 0.0) and (Frac(y) = 0.0)
          then if Odd (Round (y))
          then xXPowerY := -xXPowerY (-x,y)
          else xXPowerY :=  xXPowerY (-x,y)
        else RunError (errorInvalidFloatOperation);
end;
{*********************************************************}
function XPowerN (x: float; n: integer): float;

const maxPower=200;

var   temp: float;

begin
  if      (n = 0)   then XPowerN := 1.0     {x^0 = 1}
  else if (x = 0.0) then XPowerN := 0.0     {0^n = 0}
  else begin
    temp := n * ln (x);
    if (temp < -maxExp) then temp := -maxExp
    else if (temp > maxExp) then temp := maxExp;
    XPowerN := exp (temp);
  end;
end;
{*********************************************************}
{
{ Aufruf:      XRootY (x,y)
{
{ Funktion:    Berechnet y-ten Wurzel aus x
{
{ Formel:      XRootY = XPowerY (x, 1 / y)    [x > 0]
{
{ Literatur:   Bartsch-84, Seite 164
{
{ Versorgung:  x: float;   Argument
{              y: float;   Wurzelzahl
{
{ Rckgabe:   XRootY: float;   y-te Wurzel aus x
{
{---------------------------------------------------------}
function XRootY (x,y: float): float;

begin
  if x < 0.0
  then XRootY := SetMathError (mInvalid, 'XRootY: x < 0.0')
  else XRootY := XPowerY (x, 1.0 / y);
end;
{*********************************************************}
{
{ Aufruf:      tan (alpha)
{
{ Funktion:    Berechnet Tangens des Winkels alpha
{
{ Literatur:   Bartsch-84, Seite 168
{
{ Formel:      tan (alpha) = sin (alpha) / cos (alpha)
{
{ Versorgung:  alpha: float;   Winkel [rad]
{
{ Rckgabe:    tan: float;     Tangens des Winkels
{
{---------------------------------------------------------}
function tan (alpha: float): float;

var   cosAlpha: float;

begin
  cosAlpha := cos (alpha);
  if cosAlpha = 0.0
  then tan := SetMathError (mOverflow, 'tan: cos(alpha) = 0')
  else tan := sin (alpha) / cosAlpha;
end;
{*********************************************************}
{
{ Aufruf:      cot (alpha)
{
{ Funktion:    Berechnet Cotangens des Winkels alpha
{
{ Literatur:   Bartsch-84, Seite 168
{
{ Formel:      cot (alpha) = cos (alpha) / sin (alpha)
{
{ Versorgung:  alpha: float;   Winkel [rad]
{
{ Rckgabe:    cot: float;     Cotangens des Winkels
{
{---------------------------------------------------------}
function cot (alpha: float): float;

var   sinAlpha: float;

begin
  sinAlpha := sin (alpha);
  if sinAlpha = 0.0
  then cot := SetMathError (mOverflow, 'cot: sin(alpha) = 0')
  else cot := cos (alpha) / sinAlpha;
end;
{*********************************************************}
{
{ Aufruf:      arcSin (y)
{
{ Funktion:    Berechnet Umkehrfunktion zu Sinus
{
{ Literatur:   Bartsch-84, Seite 175
{
{ Formel:      arcSin (y) = arcTan (y / sqrt (1 - y^2))
{                                             [-1 <= y <= 1]
{ Versorgung:  y: float;   Winkel [rad]
{
{ Rckgabe:    arcSin: float;  Winkel  [-Pi/2 <= arcSin <= Pi/2]
{
{---------------------------------------------------------}
function arcSin (y: float): float;

var   sqrY: float;

begin
  sqrY := y * y;
  if sqrY > 1.0
  then arcSin := SetMathError (mOverflow, 'arcSin: abs (y) > 1')
  else arcSin := arcTan2 (y, sqrt (1.0 - sqrY));
end;
{*********************************************************}
{
{ Aufruf:      arcCos (y)
{
{ Funktion:    Berechnet Umkehrfunktion zu Cosinus
{
{ Literatur:   Bartsch-84, Seite 187, Gellert-72, Seite 155
{
{ Formel:      arcCos (y) = Pi / 2 - arcSin (y)
{                                          [-1 <= y <= 1]
{ Versorgung:  y: float;   Winkel [rad]
{
{ Rckgabe:    arcCos: float;    Winkel [0 <= arcCos <= Pi]
{
{---------------------------------------------------------}
function arcCos (y: float): float;

var   sqrY: float;

begin
  sqrY := y * y;
  if sqrY > 1.0
  then begin
    arcCos := SetMathError (mInvalid, 'arcCos: abs (y) > 1');
    if y < 1.0 then arcCos := rad180;
  end
  else arcCos := rad90 - arcTan2 (sqrt (1.0 - sqrY), y);
end;
{*********************************************************}
{
{ Aufruf:      arcTan2 (x,y)
{
{ Funktion:    Berechnet Umkehrfunktion zu Tangens mit
{              2 Argumenten bzw. Berechnen des Winkels
{              alpha eines Dreiecks mit Ankathete x
{              und Gegenkathete y
{
{ Literatur:   Bartsch-84, Seite 187
{
{ Formel:      arcTan2 (x,y) = arcTan (y / x) [x # 0]   /|
{              arcTan2 (0,+y) = +Pi / 2 (+90)        /  | y
{              arcTan2 (0,-y) = -Pi / 2 (-90)      /_)__|
{                                                      x
{ Versorgung:  x: float;   Ankathete des rechtwink. Dreiecks
{              y: float;   Gegenkathete des rechtw. Dreiecks
{
{ Rckgabe:    arcTan2: float; Winkel [-Pi/2 <= arcTan2 <= Pi/2]
{
{---------------------------------------------------------}
function arcTan2 (x, y: float): float;

begin
  if abs(x) < 1E-20
  then begin
    if y = 0.0
    then arcTan2 := SetMathError (mInvalid, 'arcTan2: x = 0, y = 0')
    else if y >= 0.0
    then arcTan2 := +rad90   {+90}
    else arcTan2 := -rad90;  {-90}
  end
(*
  else if abs(y) < 1E-20
  then if x >= 0
  then arcTan2 := 0         {  0}
  else arcTan2 := Pi        {180}
*)
  else if x >= 0.0
  then arcTan2 := arcTan (y / x)          {-90..90}
  else arcTan2 := arcTan (y / x) + Pi;    {90..270}
end;
{*********************************************************}
{
{ Aufruf:      arcCot (x)
{
{ Funktion:    Berechnet Umkehrfunktion zu Cotangens
{
{ Literatur:   Bartsch-84, Seite 188
{
{ Formel:      arcCot (x) = Pi / 2 - arcTan (x)
{
{ Versorgung:  x: float;   Argument
{
{ Rckgabe:    arcCot: float;    Winkel    [0 <= arcCot <= Pi]
{
{---------------------------------------------------------}
function arcCot (x: float): float;

begin
  arcCot := rad90 - arcTan (x);
end;
{*********************************************************}
{
{ Aufruf:      degree (angle)
{
{ Funktion:    Umrechnung eines Winkels von Radiant in Altgrad
{
{ Literatur:   Bartsch-84, Seite 210
{
{ Versorgung:  angle: float;    Winkel in Radiant
{
{ Rckgabe:    degree: float;   Winkel in Altgrad
{
{---------------------------------------------------------}
function degree (angle: float): float;

begin
  degree := angle * rad;
end;
{*********************************************************}
{
{ Aufruf:      radiant (angle)
{
{ Funktion:    Umrechnung eines Winkels von Altgrad in Radiant
{
{ Literatur:   Bartsch-84, Seite 211
{
{ Versorgung:  angle: float;  Winkel in Altgrad
{
{ Rckgabe:    radiant: float;   Winkel in Radiant
{
{---------------------------------------------------------}
function radiant (angle: float): float;

begin
  radiant := angle * deg;
end;
{*********************************************************}
{
{ Aufruf:      exp10 (y)
{
{ Funktion:    Berechnet 10 hoch y
{
{ Formel:      exp10 (y) = exp(y + ln (1))
{
{ Literatur:   Bartsch-84, Seite 481
{
{ Versorgung:  y: float;   Exponent
{
{ Rckgabe:    exp10: float;   10 hoch y
{
{---------------------------------------------------------}
function exp10 (y: float): float;

begin
  exp10 := exp (y + ln10);
end;
{*********************************************************}
{
{ Aufruf:      log (x,b)
{
{ Funktion:    Berechnet Logarithmus von x zur Basis b
{
{ Literatur:   Bartsch-84, Seite 167
{
{ Formel:      log(x) = ln(x) / ln(b)   [x > 0, b > 0, b # 1.0]
{
{ Versorgung:  x: float;   Argument
{              b: float;   Basis
{
{ Rckgabe:    log: float;   Logarithmus von x zur Basis b
{
{---------------------------------------------------------}
function log (x,b: float): float;

begin
  if x <= 0.0
  then log := SetMathError (mOverflow, 'log: x <= 0')
  else
    if (b <= 0.0) or (b = 1.0)
    then log := SetMathError (mOverflow, 'log: b <= 0  or  b = 1')
    else log := ln (x) / ln (b);
end;
{*********************************************************}
{
{ Aufruf:      log10 (x)
{
{ Funktion:    Berechnet Logarithmus von x zur Basis 10
{
{ Literatur:   Bartsch-84, Seite 56
{
{ Formel:      log10 (x) = ln (x) / ln (10)    [x > 0]
{
{ Versorgung:  x: float;   Argument
{
{ Rckgabe:    log10: float;   Logarithmus von x zur Basis 10
{
{---------------------------------------------------------}
function log10 (x: float): float;

begin
  if x <= 0.0
  then log10 := SetMathError (mOverflow, 'log10: x <= 0')
  else log10 := ln (x) * rln10;
end;

{*********************************************************}
{
{ Aufruf:      signum (x)
{
{ Funktion:    Ermittelt Vorzeichen einer reellen Zahl
{
{ Versorgung:  x: float;   Argument
{
{ Rckgabe:    signum: float;   Vorzeichen der bergebenen Zahl
{
{---------------------------------------------------------}
function signum (x: float): float;

begin
  if x = 0.0
  then signum := 0.0
  else
    if x > 0.0
    then signum := (+1.0)
    else signum := (-1.0);
end;
{*********************************************************}
{
{ Aufruf:      frac (x)
{
{ Funktion:    Berechnet Nachkommateil einer reellen Zahl
{
{ Formel:      frac (x) = x - int (x)
{
{ Versorgung:  x: float;   Argument
{
{ Rckgabe:    frac: float;   Nachkommateil der bergebenen Zahl
{
{---------------------------------------------------------}
function frac (x: float): float;

begin
  frac := x - trunc(x);
end;
{*********************************************************}
{
{ Aufruf:      fix (x,n)
{
{ Funktion:    Beschneidet reelle Zahl x auf n Nachkommastellen
{
{ Formel:      fix (x) = int (x * 10^n) * log10(n)
{
{ Versorgung:  x: float;   Argument
{
{ Rckgabe:    fix: float;   Nachkommateil der bergebenen Zahl
{
{---------------------------------------------------------}
function fix (x: float; n: word): float;

var   r: float;

begin
  r := n;
  fix := int (x * exp10 (r)) * log10 (r);
end;

{*********************************************************}
{
{ Aufruf:      fak (n)
{
{ Funktion:    Berechnet Fakultt der Zahl n
{              bzw. der Permutationen von n Elementen
{
{ Literatur:   Bartsch-84, Seite 58
{
{ Formel:      n! = n * (n! - 1)        0! = 1       1! = 1
{
{ Versorgung:  n: word;   natrliche Zahl (0 <= n <= 34)
{
{ Rckgabe:    fak: float;   Fakultt von n
{
{---------------------------------------------------------}
function fak (n: word): float;

var   d,i: word;   f: float;

begin
  if n > 34
  then fak := SetMathError (mOverflow, 'fak: n > 34')
  else begin
    f := 1.0;                          {  0! }
    d := n div 8;
    case d of
      1: f := 40320.0;                 {  8! }
      2: f := 2.092278989E13;          { 16! }
      3: f := 6.204484016E23;          { 24! }
      4: f := 2.631308368E35;          { 32! }
    end;
    for i := d * 8 + 1 to n
    do f := f * i;
    fak := f;
  end;
end;
{*********************************************************}
{
{ Aufruf:      variations (k,n)
{
{ Funktion:    Berechnet Variationen von k Elementen in allen
{              mglichen Reihenfolgen aus n gegebenen Elementen.
{
{ Literatur:   Bartsch-84, Seite 72
{
{ Formel:      V(k,n) = n! / (n - k)!    [k <= n]
{
{ Versorgung:  k,n: word;   k aus n Elementen
{
{ Rckgabe:    variations: float;   Anzahl mglicher Variationen
{
{---------------------------------------------------------}
function variations (k,n: word): float;

begin
  if k > n
  then variations := SetMathError (mInvalid, 'variations: k > n')
  else variations := fak (n) / fak (n - k);
end;
{*********************************************************}
{
{ Aufruf:      combinations (k,n)
{
{ Funktion:    Berechnen der Kombinationen von k Elementen in
{              beliebiger Reihenfolge aus n gegebenen Elementen.
{
{ Literatur:   Bartsch-84, Seite 73
{
{ Formel:      C(k,n) = n! / (k! * (n - k)!)
{                     = V(k,n) / k!   [k <= n]
{
{ Versorgung:  k,n: word;   k aus n Elementen
{
{ Rckgabe:    combinations: float;   mgliche Kombinationen
{
{---------------------------------------------------------}
function combinations (k,n: word): float;

begin
  if k > n
  then combinations := SetMathError (mInvalid, 'combinations: k > n')
  else combinations := fak (n) / (fak (k) * fak (n - k));
end;

{*********************************************************}
{
{ Aufruf:      ggt (n1,n2)
{
{ Funktion:    Berechnet grssten gemeinsamen Teiler
{              zweier ganzzahliger Zahlen (GGT (0,0) = 0)
{
{ Literatur:   Bartsch-84, Seite ???
{
{ Versorgung:  n1,n2: longint;   Zahlen, fr ggt Berechnung
{
{ Rckgabe:    ggt: longint;     grsster gemeinsame Teiler
{
{---------------------------------------------------------}
function ggt (n1, n2: longint): longint;

begin
  n1 := abs (n1);
  n2 := abs (n2);
  if (n1 = 0) and (n2 = 0)
  then ggt := 0
  else begin
    if n2 <> 0
    then repeat
      n1 := n1 mod n2;
      if n1 > 0
      then n2 := n2 mod n1;
    until (n1 = 0) or (n2 = 0);
    ggt := n1 + n2;
  end;
end;
{*********************************************************}
{
{ Aufruf:      kgv (n1,n2)
{
{ Funktion:    Berechnet kleinstes gemeinsames Vielfaches
{              zweier ganzzahliger Zahlen
{
{ Literatur:   Bartsch-84, Seite 49
{
{ Versorgung:  n1,n2: longint;   Zahlen fr kgv Berechnung
{
{ Rckgabe:    kgv: longint;    kleinstes gemeinsames Vielfaches
{
{---------------------------------------------------------}
function kgv (n1, n2: longint): longint;

var   n: longint;

begin
  n := ggt (n1,n2);
  if n = 0
  then kgv := 0
  else kgv := (n1 div n) * n2;
end;
{*********************************************************}
{
{ Aufruf:      fib (n)
{
{ Funktion:    Berechnet Fibonacci-Zahl (n <= 185)
{
{ Formel:      fib(0)=1  fib(1)=1  fib(n)=fib(n-1)+fib(n-2)
{
{ Literatur:   Gellert-72, Seite 389
{
{ Versorgung:  n: word;   Zahl fr Fibonacci Berechnung
{
{ Rckgabe:    fib: float;    Fibonacci-Zahl
{
{---------------------------------------------------------}
function Fib (n: integer): float;

const fs = sizeof (float) * 2;

var   f: array[0..2] of float;   ni: longint;

begin
  if      n =   0 then Fib := 0.0
  else if n > 185 then Fib := maxFloat
  else begin
    f[0] := 0;
    f[1] := 1;
    f[2] := 1;
    ni := 1;
    while ni < n
    do begin
      f[2] := f[0] + f[1];
      move (f[1], f[0], fs);  {f0:=f1;  f1:=f2;}
      inc (ni);
    end;
    Fib := f[2];
  end;
end;
{*********************************************************}
procedure CalcSinusTab;

var   xi: word;

begin
  for xi := 0 to 255
  do begin
    sinS[xi] := round (sin(pi*xi/128) * divSCS);
    cosS[xi] := round (cos(pi*xi/128) * divSCS);
    sinB[xi] := sinS[xi] + offSCS;
    cosB[xi] := cosS[xi] + offSCS;
{
    writeln (tst, xi:3, sinS[xi]:6,sinB[xi]:6,cosS[xi]:6,cosB[xi]:6);
}
  end;
end;
{*********************************************************}
begin
  CalcSinusTab;
end.
