//*******************************************************************
//
//  UNIT:     R o t o Z o o m                    02.06.99,  29.07.99
//
//  rotation and zooming of picture with integer arithmetic with Delphi-4
//
//-------------------------------------------------------------------
//
//  StartZoomRotation   start zoom rotation demo
//  ZoomRotation        rotating and zooming bitmap in realtime
//
//*******************************************************************
{$Q+}   // must be set for correct integer arithmetic, don't know why
unit RotoZoom;

interface

uses  SysUtils, Graphics;

function StartZoomRotation: string;
procedure ZoomRotation;

//=========================================================
implementation

uses  Graph, Palette;

const picName = 'rot2.bmp';

var   rotBm: tBitmap = nil;

//=========================================================
// Picture Rotation Demo
//=========================================================

const slen = 1023;
      rotatePicX = 256;
      rotatePicY = 128;
      rotatePicS = rotatePicX * rotatePicY;
      rcnt: longint = 0;

type  pByte = ^byte;

var   s: array[0..255]of smallInt;
      t: array[0..slen]of smallInt;
      loadError: string[40];

//---------------------------------------------------------
// initialize sinus tables and load rotation picture
//---------------------------------------------------------
procedure InitZoomRotation;

var  ai: integer;

begin
  for ai := 0 to 255
  do s[ai]:=-333+round(sin(pi*ai/128)*128);

  for ai := 0 to 1023
  do t[ai]:=round(sin(pi*ai/512)*512);
end;
//---------------------------------------------------------
function StartZoomRotation: string;
begin
  result := 'fast zoom rotation';
  SetGraphMode (pf8Bit);
  // load picture with 256*128/256 for rotor zoom
  if rotBm = nil
  then if not LoadBitmap (rotBm, picName)
  then loadError := 'loading error picture ' + picName
  else if rotBm = nil
  then loadError := 'creation error picture ' + picName
  else if (rotBm.height <> 128)
  or (rotBm.width <> 256)
  then OutTextXY (5,5,'picture dimension error ' + picName);

  CopyBitmapPalette (rotBm.palette);
end;
//---------------------------------------------------------
// do picture rotation and zooming with integer arithmetic
// input:      x1,y1,x2,y2: destination drawing rectangle
//   global:   rotBm: rotation bitmap
//---------------------------------------------------------
procedure RotatePicture (src: pByteArray;
                         x1,y1,x2,y2: integer);

var   xm, ym, xi,yi, u1,v1, u,v, du,dv: longint;
      dst: pByte;
begin
  xm := (rotBm.width - 1);
  ym := (rotBm.height - 1) shl 8;
  inc (rcnt);
  u  := t[(rcnt) and slen];
  du := t[(u+rcnt)    and slen] * s[byte(rcnt+22)];
  dv := t[(u+rcnt+555)and slen] * s[byte(rcnt+44)];
  u1 := -du;
  v1 := +dv;
  for yi := y1 to y2
  do begin
    inc (u1,du);
    inc (v1,dv);
    u := u1;
    v := v1;
    dst := GetLinePointer (yi);
    for xi := x1 to x2
    do begin
      dst^ := src^[(u shr 17) and xm + (v shr 9) and ym];
      inc (dst);
      inc (u,dv);
      dec (v,du);
    end;
  end;
end;
//*********************************************************
// example of fast pixel drawing using tBitmap.ScanLine[..]
// with 256 colors and color palette
//---------------------------------------------------------
procedure ZoomRotation;
begin
  if loadError <> ''
  then OutTextXY (5,5, loadError)
  else RotatePicture (rotBm.scanline[rotBm.height-1]
                     ,0,0, GetMaxX,GetMaxY);
end;

initialization
  InitZoomRotation;

finalization
  if rotBm <> nil then rotBm.Free;

end.

