//*******************************************************************
//
//  UNIT:     W A V Y                            11.10.98,  08.09.99
//
//-------------------------------------------------------------------
//
//  StartWavy       start interference of 2 sinus waves
//  AnimWavy        interference animation
//
//*******************************************************************
unit Wavy;

interface

uses  SysUtils, Graphics;

function StartWavy: string;
procedure AnimWavy;

//=========================================================
implementation

uses  Graph, Palette;

const maxX = 255;   dimX = maxX + 1;  {must be 2^n !!!}
      maxY = 255;   dimY = maxY + 1;
      oversizeY = maxY + 10;
      mask = dimx*dimy-1;

var   wavBm: tBitmap = nil;
      picNo: integer = 0;

//*********************************************************
function StartWavy: string;
begin
  result := 'overlapping of two sinus waves';
  SetGraphMode (pf8Bit);
  CreateColorMap (221, 171, 123);
end;
//*********************************************************
// create dimX * oversizeY / 256 2d sine wave field
//---------------------------------------------------------
procedure InitWavy;
var   xi,yi: longint;
      df: double;
      linePtr: pByteArray;
begin
  wavBM := TBitmap.Create;
  wavBM.PixelFormat := pf8Bit;
  wavBm.height := oversizeY+1;
  wavBm.width  := dimX;
  for yi := 0 to overSizeY
  do begin
    linePtr := wavBm.ScanLine[yi];
    for xi := 0 to maxX
    do begin
      df := sqrt (sqr(128-xi) + sqr(128-yi));
      linePtr^[xi] := round(sin(df*0.25)*63) + 64;
    end;
  end;
end;

//*********************************************************
procedure DrawWavy;
var   xi,yi, d1,d2: longint;
      p0,p1,p2: ^byte;   {Fensterposition}
      v1: word;
      w1,w2,wb: ^word;
begin
  inc (picNo);
  p0 := wavBm.scanline[oversizeY];
  d1 := round((cos(picNo*0.041)*50)+50)*dimX     {offset 1.source}
       +round((sin(picNo*0.037)*50)+50);
  d2 := round((sin(picNo*0.021)*50)+50)*dimX     {offset 2.source}
       +round((cos(picNo*0.053)*50)+50);

  for yi := 0 to (GetMaxY div 2)
  do begin
    p1 := pointer (longint (p0) + d1);
    p2 := pointer (longint (p0) + d2);
    w1 := GetLinePointer(yi*2);
    w2 := GetLinePointer(yi*2+1);
    wb := w1;
    for xi := 0 to GetMaxX div 2
    do begin
      v1 := byte(p1^ + p2^);
      wb^ := v1 + v1 shl 8;
      inc (wb);
      inc (p1);
      inc (p2);
    end;
    move (w1^, w2^, pageX);       // copy line
    d1 := (d1+dimX) and mask;     //correct offset to next line
    d2 := (d2+dimX) and mask;
  end;
end;

//*********************************************************
// display transparent shadowed help text
//---------------------------------------------------------
procedure DisplayHelp;
var   h: word;
begin
  TextColor (clWhite);
  h := TextHeight;
  ShadowTextXY (10,1*h, '<ret> create new colors');
end;
//*********************************************************
procedure AnimWavy;
begin
  DrawWavy;

  if animHelp then DisplayHelp;
  case cb of
    #13: CreateColorMap (221, 171, 123);
  end;
  cb := #0;
end;

//*********************************************************
initialization
  InitWavy;

finalization
  if wavBm <> nil then wavBm.Free;

end.
